package recorder

import (
	"context"

	"go.opentelemetry.io/otel/attribute"
	"go.opentelemetry.io/otel/codes"
	"k8s.io/utils/ptr"

	pb "gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/api/v1alpha1/weather"
	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/weather"
)

// Returns last requested number of weather updates
// If negative number given, will return all weather observations
func (w *WeatherRecorder) Get(ctx context.Context, req *pb.GetWeatherRequest) (
	[]*weather.WeatherUpdate, error,
) {
	if req == nil {
		req = &pb.GetWeatherRequest{Limit: ptr.To(int32(-1))}
	}

	if req.Limit == nil || *req.Limit < 0 {
		req.Limit = ptr.To(int32(-1))
	} else if *req.Limit == 0 {
		req.Limit = ptr.To(int32(1))
	}

	var filterSN, filterST string
	if req.Opts != nil {
		filterSN = req.Opts.GetStationName()
		filterST = req.Opts.GetStationType()
	}

	ctx, span := w.tracer.Start(ctx, "getWeatherRecorder")
	span.SetAttributes(
		attribute.String("stationNameFilter", filterSN),
		attribute.String("stationTypeFilter", filterST),
		attribute.Int("last", int(*req.Limit)),
		attribute.Int("currentSize", w.Count(ctx)),
	)
	defer span.End()

	updates, err := w.recorder.Get(ctx, req)
	if err != nil {
		span.RecordError(err)
		span.SetStatus(codes.Error, err.Error())
	} else {
		span.SetStatus(codes.Ok, "")
	}

	return updates, err
}

// Returns count of retained weather updates
func (w *WeatherRecorder) Count(ctx context.Context) int {
	ctx, span := w.tracer.Start(ctx, "countWeatherRecorder")
	defer span.End()

	count := w.recorder.Count(ctx)
	span.SetAttributes(attribute.Int("count", count))

	span.SetStatus(codes.Ok, "")
	return count
}
