package recorder

import (
	"context"

	"go.opentelemetry.io/otel/attribute"
	"go.opentelemetry.io/otel/trace"

	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/ambient/config"
	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/util"
	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/weather"
)

func (w *WeatherRecorder) Set(ctx context.Context, u *weather.WeatherUpdate) error {
	if u.StationConfig == nil {
		u.StationConfig = &config.WeatherStation{
			Name:                "unregistered",
			Equipment:           "unknown",
			ProxyToAWN:          false,
			ProxyToWunderground: false,
			KeepMetrics:         []string{},
		}
	}

	ctx, span := w.tracer.Start(ctx, "setRecorderUpdate", trace.WithAttributes(
		attribute.String("stationName", u.StationConfig.Name),
		attribute.String("stationType", util.DerefStr(u.StationType)),
		attribute.String("stationEquipment", u.StationConfig.Equipment),
	))
	defer span.End()

	return w.recorder.Set(ctx, u)
}
