package memory

import (
	"context"
	"errors"

	"go.opentelemetry.io/otel/attribute"
	"go.opentelemetry.io/otel/codes"

	pb "gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/api/v1alpha1/weather"
	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/util"
	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/weather"
)

func (r *MemoryRecorder) Get(ctx context.Context, req *pb.GetWeatherRequest) (
	[]*weather.WeatherUpdate, error,
) {
	_, span := r.tracer.Start(ctx, "memoryRecorder.Get")
	defer span.End()

	r.RLock()
	defer r.RUnlock()

	span.AddEvent("acquired lock on recorder cache")

	limit := util.GetLimitFromReq(req)
	if r.count() == 0 {
		err := errors.New("no recorded updates to get")
		span.RecordError(err)
		return nil, err
	} else if limit > 0 && r.count() <= limit {
		span.RecordError(errors.New("requested more updates than recorded"))
	}

	updates := r.getUpdatesFromReq(req)
	span.AddEvent("request limit/opts applied to updates")

	span.SetAttributes(attribute.Int("retrieved", len(updates)))
	span.SetStatus(codes.Ok, "")

	return updates, nil
}

func (r *MemoryRecorder) getUpdatesFromReq(req *pb.GetWeatherRequest) []*weather.WeatherUpdate {
	if req.Opts == nil {
		return util.LimitUpdates(r.updates, util.GetLimitFromReq(req))
	}

	return util.ApplyOptsToUpdates(r.updates, util.GetLimitFromReq(req), req.Opts)
}
