package memory

import (
	"context"
	"sync"

	"go.opentelemetry.io/otel/trace"

	"gitea.libretechconsulting.com/rmcguire/go-app/pkg/otel"

	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/weather"
	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/weather/recorder/recorders"
)

const (
	DEF_RETAIN_LAST = 120
	NAME            = "memory recorder"
)

type MemoryRecorder struct {
	baseCtx context.Context
	updates []*weather.WeatherUpdate
	tracer  trace.Tracer
	keep    int
	*sync.RWMutex
}

// No meaningful health check to do here
func (r *MemoryRecorder) Ping(ctx context.Context) error {
	return nil
}

func (r *MemoryRecorder) Init(ctx context.Context, opts *recorders.RecorderOpts) {
	if opts.RetainLast < 1 {
		opts.RetainLast = DEF_RETAIN_LAST
	}

	r.updates = make([]*weather.WeatherUpdate, 0, opts.RetainLast)
	r.keep = opts.RetainLast
	r.baseCtx = opts.BaseCtx
	r.RWMutex = &sync.RWMutex{}
	r.tracer = otel.GetTracer(r.baseCtx, "memoryRecorder")
}

func (r *MemoryRecorder) Name() string { return NAME }
