package memory

import (
	"context"

	"go.opentelemetry.io/otel/attribute"
	"go.opentelemetry.io/otel/trace"

	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/weather"
)

func (r *MemoryRecorder) Set(ctx context.Context, u *weather.WeatherUpdate) error {
	r.Lock()
	defer r.Unlock()

	ctx, span := r.tracer.Start(ctx, "memoryRecorder.Set")
	span.SetAttributes(
		attribute.Int("countWeatherUpdates", r.Count(ctx)),
		attribute.Int("keepUpdates", r.keep),
	)
	defer span.End()

	return r.set(ctx, u)
}

func (r *MemoryRecorder) set(ctx context.Context, u *weather.WeatherUpdate) error {
	span := trace.SpanFromContext(ctx)

	if len(r.updates) > r.keep {
		r.updates = r.updates[1:]
		span.AddEvent("trimmed recorded updates by 1")
	}

	r.updates = append(r.updates, u)
	span.AddEvent("recorded weather update")

	return nil
}
