package redis

import (
	"context"

	"go.opentelemetry.io/otel/attribute"
	"go.opentelemetry.io/otel/codes"
	"go.opentelemetry.io/otel/trace"
)

func (r *RedisRecorder) Count(ctx context.Context) int {
	ctx, span := r.tracer.Start(ctx, "redisRecorder.count")
	defer span.End()

	r.RLock()
	defer r.RUnlock()

	return r.count(ctx)
}

func (r *RedisRecorder) count(ctx context.Context) int {
	ctx, span := r.tracer.Start(ctx, "redisRecorder.count.redis", trace.WithAttributes(
		attribute.String("updatesKey", r.Key())))
	defer span.End()

	count, err := r.redis.LLen(ctx, r.Key()).Result()
	if err != nil {
		span.RecordError(err)
		span.SetStatus(codes.Error, err.Error())
		r.log.Err(err).Send()
		return int(count)
	}

	span.SetAttributes(attribute.Int64("updatesCount", count))
	span.SetStatus(codes.Ok, "")

	return int(count)
}
