package weather

import (
	"time"

	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/ambient/config"
)

// Stable intermediate struct containing superset of fields
// between AWN and Wunderground style updates from Ambient devices
type WeatherUpdate struct {
	DateUTC         *time.Time             `json:"dateUTC,omitempty"`
	StationConfig   *config.WeatherStation `json:"stationConfig,omitempty"`
	StationID       *string                `json:"stationID,omitempty"`
	StationType     *string                `json:"stationType,omitempty"`
	TempOutdoorF    *float64               `json:"tempOutdoorF,omitempty"`
	TempIndoorF     *float64               `json:"tempIndoorF,omitempty"`
	HumidityOudoor  *int                   `json:"humidityOudoor,omitempty"`
	HumidityIndoor  *int                   `json:"humidityIndoor,omitempty"`
	WindSpeedMPH    *float64               `json:"windSpeedMPH,omitempty"`
	WindGustMPH     *float64               `json:"windGustMPH,omitempty"`
	MaxDailyGust    *float64               `json:"maxDailyGust,omitempty"`
	WindDir         *int                   `json:"windDir,omitempty"`
	WindDirAvg10m   *int                   `json:"windDirAvg10M,omitempty"`
	UV              *int                   `json:"uv,omitempty"`
	SolarRadiation  *float64               `json:"solarRadiation,omitempty"`
	HourlyRainIn    *float64               `json:"hourlyRainIn,omitempty"`
	EventRainIn     *float64               `json:"eventRainIn,omitempty"`
	DailyRainIn     *float64               `json:"dailyRainIn,omitempty"`
	WeeklyRainIn    *float64               `json:"weeklyRainIn,omitempty"`
	MonthlyRainIn   *float64               `json:"monthlyRainIn,omitempty"`
	YearlyRainIn    *float64               `json:"yearlyRainIn,omitempty"`
	TotalRainIn     *float64               `json:"totalRainIn,omitempty"`
	Batteries       []BatteryStatus        `json:"batteries,omitempty"`
	BaromRelativeIn *float64               `json:"baromRelativeIn,omitempty"`
	BaromAbsoluteIn *float64               `json:"baromAbsoluteIn,omitempty"`
	// These fields may be calculated
	// if not otherwise set
	DewPointF  *float64 `json:"dewPointF,omitempty"`
	WindChillF *float64 `json:"windChillF,omitempty"`
	// Extra Temp+Humidity Sensors
	TempHumiditySensors []*TempHumiditySensor `json:"tempHumiditySensors,omitempty"`
	// Lightning Data
	LightningDay      *int `json:"lightningDay,omitempty"`      // Count of detections
	LightningDistance *int `json:"lightningDistance,omitempty"` // Last detection distance [kilometers]
	LightningLastTime *int `json:"lightningLastTime,omitempty"` // Last detection time
}

type TempHumiditySensor struct {
	Name     string   `json:"name,omitempty"`
	TempF    *float64 `json:"tempF,omitempty"`
	Humidity *int     `json:"humidity,omitempty"`
}

type BatteryStatus struct {
	Component string `json:"component,omitempty"`
	Status    *int   `json:"status,omitempty"`
}

// CHORE: Maintain this, used to check against
// keep and drop lists
// TODO: Use refelct/ast to generate code
const (
	FieldDateUTC           = "DateUTC"
	FieldStationType       = "StationType"
	FieldTempOutdoorF      = "TempOutdoorF"
	FieldTempIndoorF       = "TempIndoorF"
	FieldHumidityOudoor    = "HumidityOudoor"
	FieldHumidityIndoor    = "HumidityIndoor"
	FieldWindSpeedMPH      = "WindSpeedMPH"
	FieldWindGustMPH       = "WindGustMPH"
	FieldMaxDailyGust      = "MaxDailyGust"
	FieldWindDir           = "WindDir"
	FieldWindDirAvg10m     = "WindDirAvg10m"
	FieldUV                = "UV"
	FieldSolarRadiation    = "SolarRadiation"
	FieldHourlyRainIn      = "HourlyRainIn"
	FieldEventRainIn       = "EventRainIn"
	FieldDailyRainIn       = "DailyRainIn"
	FieldWeeklyRainIn      = "WeeklyRainIn"
	FieldMonthlyRainIn     = "MonthlyRainIn"
	FieldYearlyRainIn      = "YearlyRainIn"
	FieldTotalRainIn       = "TotalRainIn"
	FieldBatteries         = "Batteries"
	FieldBaromRelativeIn   = "BaromRelativeIn"
	FieldBaromAbsoluteIn   = "BaromAbsoluteIn"
	FieldDewPointF         = "DewPointF"
	FieldWindChillF        = "WindChillF"
	FieldSensorTempF       = "SensorTempF"
	FieldSensorHumidity    = "SensorHumidity"
	FieldLightningDay      = "LightningDay"
	FieldLightningDistance = "LightningDistance"
	FieldLightningLastTime = "LightningLastTime"
)

func (u *WeatherUpdate) GetStationName() string {
	if u.StationConfig != nil {
		return u.StationConfig.Name
	}
	return ""
}
